from django_glue.utils import GLUE_RESPONSE_TYPES
from django.http import JsonResponse


def generate_json_response(status, response_type: str, message_title, message_body, additional_data=None):
    if response_type not in GLUE_RESPONSE_TYPES:
        raise ValueError(f'response_type "{response_type}" is not a valid, choices are {GLUE_RESPONSE_TYPES}')

    return JsonResponse({
        'type': response_type,
        'message_title': message_title,
        'message_body': message_body,
        'data': additional_data
    }, status=status)


def generate_json_404_response():
    return generate_json_response('404', 'error', 'Request not Found',
                                  'The requested information, object or view you are looking for was not found.')



# Generated by Django 4.0.4 on 2022-08-06 03:04

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=32)),
                ('last_name', models.CharField(max_length=32)),
                ('description', models.TextField()),
                ('favorite_number', models.IntegerField()),
                ('anniversary_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('birth_date', models.DateField(default=django.utils.timezone.now)),
                ('weight_lbs', models.DecimalField(decimal_places=3, max_digits=7)),
            ],
        ),
    ]



import logging, pickle, base64, json

from django.contrib.contenttypes.models import ContentType
from django.db.models import Model
from django.db.models.query import QuerySet
from django.http import Http404

from django_glue.conf import settings

GLUE_ACCESS_TYPES = (
    'view',
    'add',
    'change',
    'delete',
)

GLUE_RESPONSE_TYPES = (
    'success',
    'info',
    'warning',
    'error',
    'debug',
)

GLUE_SESSION_TYPES = (
    'context',
    'query_set',
    'fields',
    'exclude',
)


def add_glue(request, unique_name: str, target, access: str, fields=('__all__',), exclude=('__none__',), **kwargs):
    if access in GLUE_ACCESS_TYPES:
        if isinstance(fields, (list, tuple)) and isinstance(exclude, (list, tuple)):
            if unique_name_unused(request, unique_name):
                glue_session = get_glue_session(request)

                if isinstance(target, Model):
                    content_type = ContentType.objects.get_for_model(target)

                    glue_session['context'][unique_name] = {
                        'connection': 'model_object',
                        'access': access,
                        'app_label': content_type.app_label,
                        'model': content_type.model,
                        'object_id': target.pk,
                    }

                    glue_session['context'][unique_name]['fields'] = generate_field_dict(target, fields, exclude)

                elif isinstance(target, QuerySet):
                    content_type = ContentType.objects.get_for_model(target.query.model)

                    glue_session['context'][unique_name] = {
                        'connection': 'query_set',
                        'access': access,
                        'app_label': content_type.app_label,
                        'model': content_type.model,
                    }

                    glue_session['context'][unique_name]['fields'] = generate_field_dict(target.query.model(), fields, exclude)
                    glue_session['query_set'][unique_name] = encode_query_set_to_str(target)

                else:
                    raise TypeError(f'target is not a valid type must be django Model or QuerySet')

                glue_session['fields'][unique_name] = fields
                glue_session['exclude'][unique_name] = exclude
            else:
                raise ValueError(f'unique_name "{unique_name}" is already being used.')
        else:
            raise TypeError(f'fields or exclude is not a valid type must be a list or tuple')
    else:
        raise TypeError(f'access "{access}" is not a valid, choices are {GLUE_ACCESS_TYPES}')


def camel_to_snake(string):
    return ''.join(['_' + c.lower() if c.isupper() else c for c in string]).lstrip('_')


def clean_glue_session(request):
    request.session[settings.DJANGO_GLUE_SESSION_NAME] = dict()


def decode_query_set_from_str(query_set_string):
    query = pickle.loads(base64.b64decode(query_set_string))
    decoded_query_set = query.model.objects.all()
    decoded_query_set.query = query
    return decoded_query_set


def encode_query_set_to_str(query_set):
    return base64.b64encode(pickle.dumps(query_set.query)).decode()


def generate_field_dict(model_object, fields, exclude):
    fields_dict = dict()

    model = type(model_object)

    for field in model._meta.fields:
        try:
            if field.name not in exclude or exclude[0] == '__none__':
                if field.name in fields or fields[0] == '__all__':
                    if hasattr(field, 'get_internal_type'):
                        fields_dict[field.name] = {
                            'type': field.get_internal_type(),
                            'value': getattr(model_object, field.name)
                        }

        except:
            raise f'Invalid field or exclude for model type {model.name}'

    return fields_dict


def generate_simple_field_dict(model_object, fields, exclude):
    fields_dict = generate_field_dict(model_object, fields, exclude)
    simple_fields_dict = {}

    for key, val in fields_dict.items():
        simple_fields_dict[key] = val['value']

    return simple_fields_dict


def get_fields_from_model(model):
    return [field for field in model._meta.fields]


def get_glue_session(request):
    if settings.DJANGO_GLUE_SESSION_NAME not in request.session:
        request.session[settings.DJANGO_GLUE_SESSION_NAME] = dict()
    for session_type in GLUE_SESSION_TYPES:
        if session_type not in request.session[settings.DJANGO_GLUE_SESSION_NAME]:
            request.session[settings.DJANGO_GLUE_SESSION_NAME][session_type] = dict()

    return request.session[settings.DJANGO_GLUE_SESSION_NAME]


def glue_access_check(access, access_level):
    if GLUE_ACCESS_TYPES.index(access) >= GLUE_ACCESS_TYPES.index(access_level):
        return True
    else:
        return False


def run_glue_method(request, model_object, method):
    if hasattr(model_object, method):
        getattr(model_object, method)(request)


def process_and_save_form_values(model_object, form_values_dict):
    logging.warning(f'{model_object = }')
    for key, val in form_values_dict.items():
        if key != 'id':
            model_object.__dict__[key] = val
    model_object.save()
    logging.warning(f'{model_object = }')


def process_and_save_field_value(model_object, field, value):
    logging.warning(f'{field = } {value = }')
    model_object.__dict__[field] = value
    model_object.save()


def unique_name_unused(request, unique_name):
    if unique_name in get_glue_session(request)['context']:
        return False
    else:
        return True







import logging, json

from django import template
from django.utils.safestring import mark_safe

from django_glue.conf import settings

register = template.Library()


@register.inclusion_tag('django_glue/django_glue.html', takes_context=True)
def glue_init(context):
    return context


@register.simple_tag(takes_context=True)
def glue_context_data_str(context):
    return mark_safe(str(json.dumps(context[settings.DJANGO_GLUE_CONTEXT_NAME])))




import logging, random

from django.views.generic import TemplateView

from tests.models import TestModel
from django_glue.utils import add_glue

FIRST_NAME_TUPLE = (
    'Fred',
    'Jane',
    'Janet',
    'Johnny',
    'Nathan',
    'Robert',
    'Ted',
)

LAST_NAME_TUPLE = (
    'Doe',
    'Hansen',
    'Johnson',
    'Mancal',
    'Smith',
    'Waldern',
    'Wilson',
)

DESCRIPTION_WORD_TUPLE = (
    'I',
    'a',
    'are'
    'artsy',
    'awesome',
    'boring',
    'can',
    'cat',
    'cool',
    'dog',
    'fast',
    'finished',
    'fly',
    'handy',
    'has',
    'has',
    'have',
    'it',
    'loves cars',
    'need',
    'she',
    'started',
    'stupid',
    'taco',
    'there',
    'to',
    'ugly',
    'want',
    'when',
    'when',
    'wood carving'
    'you',
)


class TestView(TemplateView):
    template_name = 'test.html'

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**kwargs)

        # TestModel.objects.all().delete()
        # logging.warning('Purged database of testing objects.')

        new_description = ''
        for x in range(10):
            new_description += f'{DESCRIPTION_WORD_TUPLE[random.randint(0, (len(DESCRIPTION_WORD_TUPLE) - 1))]} '

        test_model_object = TestModel.objects.create(
            first_name=FIRST_NAME_TUPLE[random.randint(0, (len(FIRST_NAME_TUPLE) - 1))],
            last_name=LAST_NAME_TUPLE[random.randint(0, (len(LAST_NAME_TUPLE) - 1))],
            description=new_description,
            favorite_number=random.randint(0, 999),
            weight_lbs=round(random.uniform(80.0, 400.0), 1),
        )

        exclude_test_model = TestModel.objects.all().order_by('-id')[:5]
        TestModel.objects.exclude(pk__in=exclude_test_model).delete()

        # test_model_object = TestModel.objects.all().latest('id')
        logging.warning(f'Added TestModel object.')

        add_glue(self.request, 'test_model_1', test_model_object, 'change', exclude=('birth_date', 'anniversary_datetime'))
        add_glue(self.request, 'test_model_2', test_model_object, 'change', exclude=('birth_date', 'anniversary_datetime'))
        add_glue(self.request, 'test_model_3', test_model_object, 'delete', exclude=('birth_date', 'anniversary_datetime'))
        add_glue(self.request, 'test_model_4', test_model_object, 'change', exclude=('birth_date', 'anniversary_datetime'))
        logging.warning('Added model object glue for TestModel Object in write mode')

        add_glue(self.request, 'test_query_1', TestModel.objects.filter(id__gte=1).filter(id__lte=10000), 'change', exclude=('birth_date', 'anniversary_datetime'))
        add_glue(self.request, 'test_query_2', TestModel.objects.filter(id__gte=1).filter(id__lte=10000), 'add', exclude=('birth_date', 'anniversary_datetime'))
        add_glue(self.request, 'test_query_3', TestModel.objects.filter(id__gte=1).filter(id__lte=10000), 'delete', exclude=('birth_date', 'anniversary_datetime'))
        logging.warning('Added model query set glue for TestModel Object in read mode')

        return context_data



from django.db import models
from django.utils.timezone import now


class TestModel(models.Model):
    first_name = models.CharField(max_length=32)
    last_name = models.CharField(max_length=32)
    description = models.TextField()
    favorite_number = models.IntegerField()
    anniversary_datetime = models.DateTimeField(default=now)
    birth_date = models.DateField(default=now)
    weight_lbs = models.DecimalField(max_digits=7, decimal_places=3)

    def __str__(self):
        return f'{self.first_name} {self.last_name}'

    def django_glue_create(self, request):
        pass

    def django_glue_update(self, request):
        pass

    def django_glue_delete(self, request):
        pass

    def django_glue_view(self, request):
        pass

